@@ -18,7 +18,7 @@
  * Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2008 Novell, Inc.
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
@@ -166,12 +166,26 @@ fetch_connections_done (DBusGProxy *proxy,
 	int i;
 
 	if (error) {
-		g_warning ("%s: error fetching %s connections: (%d) %s.",
-		           __func__,
-		           priv->scope == NM_CONNECTION_SCOPE_USER ? "user" : "system",
-		           error->code,
-		           error->message ? error->message : "(unknown)");
+		gboolean is_spawn_error = FALSE;
+
+		/* Don't warn if the user settings service wasn't running since that's
+		 * just annoying when running headless.
+		 */
+		if (   g_error_matches (error, DBUS_GERROR, DBUS_GERROR_SERVICE_UNKNOWN)
+		    || g_error_matches (error, DBUS_GERROR, DBUS_GERROR_NAME_HAS_NO_OWNER))
+			is_spawn_error = TRUE;
+
+		if (!is_spawn_error || priv->scope == NM_CONNECTION_SCOPE_SYSTEM) {
+			g_warning ("%s: error fetching %s connections: (%d) %s.",
+				       __func__,
+				       priv->scope == NM_CONNECTION_SCOPE_USER ? "user" : "system",
+				       error->code,
+				       error->message ? error->message : "(unknown)");
+		}
 		g_clear_error (&error);
+
+		/* We tried to read connections and failed */
+		g_signal_emit_by_name (self, NM_SETTINGS_INTERFACE_CONNECTIONS_READ);
 		return;
 	}
 
@@ -414,6 +428,7 @@ constructor (GType type,
 	                                         NM_DBUS_PATH_SETTINGS,
 	                                         NM_DBUS_IFACE_SETTINGS);
 	g_assert (priv->proxy);
+	dbus_g_proxy_set_default_timeout (priv->proxy, G_MAXINT);
 
 	dbus_g_proxy_add_signal (priv->proxy, "NewConnection",
 	                         DBUS_TYPE_G_OBJECT_PATH,
